sparse_to_dense

Function sparse_to_dense 

pub fn sparse_to_dense(
    ctx: &Context,
    matrix_a: &SparseMatrixDescriptor<'_>,
    matrix_b: &mut DenseMatrixDescriptor<'_>,
    algorithm: SparseToDenseAlgorithm,
    external_buffer: Option<DevicePtr>,
) -> Result<()>
Converts matrix_a in CSR, CSC, or COO format into its dense representation matrix_b. Blocked-ELL is not currently supported.

sparse_to_dense_buffer_size returns the size of the workspace needed by sparse_to_dense.

sparse_to_dense supports the following index type for representing matrix_a:

sparse_to_dense supports the following data types:

sparse_to_dense supports the following algorithm:

AlgorithmNotes
SparseToDenseAlgorithm::DefaultDefault algorithm.

sparse_to_dense has the following properties:

  • Requires no extra storage.
  • Supports asynchronous execution.
  • Provides deterministic (bitwise) results for each run.
  • Allows the indices of matrix_a to be unsorted.

sparse_to_dense supports the following optimizations:

  • CUDA graph capture.
  • Hardware Memory Compression.