pub struct SpGemmDescriptor { /* private fields */ }Implementations
Sourceimpl SpGemmDescriptor
impl SpGemmDescriptor
pub fn create(ctx: &Context) -> Result<Self>
pub fn num_products(&self) -> Result<usize>
pub fn as_raw(&self) -> cusparseSpGEMMDescr_t
Sourcepub unsafe fn from_raw(
handle: cusparseSpGEMMDescr_t,
ctx: &Context,
) -> Result<Self>
pub unsafe fn from_raw( handle: cusparseSpGEMMDescr_t, ctx: &Context, ) -> Result<Self>
Wraps an existing cuSPARSE SpGEMM descriptor and takes ownership of it.
Safety
handle must be a valid cusparseSpGEMMDescr_t associated with ctx.
Ownership of handle is transferred to the returned descriptor, and the
handle must not be destroyed elsewhere after calling this function.
Sourcepub fn into_raw(self) -> cusparseSpGEMMDescr_t
pub fn into_raw(self) -> cusparseSpGEMMDescr_t
Consumes the descriptor and returns the raw cuSPARSE handle without destroying it.
The caller becomes responsible for eventually destroying the returned
handle with cusparseSpGEMM_destroyDescr.