pub struct SpSmDescriptor { /* private fields */ }Implementations
Sourceimpl SpSmDescriptor
impl SpSmDescriptor
pub fn create(ctx: &Context) -> Result<Self>
pub fn as_raw(&self) -> cusparseSpSMDescr_t
Sourcepub unsafe fn from_raw(
handle: cusparseSpSMDescr_t,
ctx: &Context,
) -> Result<Self>
pub unsafe fn from_raw( handle: cusparseSpSMDescr_t, ctx: &Context, ) -> Result<Self>
Wraps an existing cuSPARSE SpSM descriptor and takes ownership of it.
Safety
handle must be a valid cusparseSpSMDescr_t associated with ctx.
Ownership of handle is transferred to the returned descriptor, and the
handle must not be destroyed elsewhere after calling this function.
Sourcepub fn into_raw(self) -> cusparseSpSMDescr_t
pub fn into_raw(self) -> cusparseSpSMDescr_t
Consumes the descriptor and returns the raw cuSPARSE handle without destroying it.
The caller becomes responsible for eventually destroying the returned
handle with cusparseSpSM_destroyDescr.