pub struct DeviceScalarMut<T> { /* private fields */ }Implementations
Sourceimpl<T> DeviceScalarMut<T>
impl<T> DeviceScalarMut<T>
Sourcepub const unsafe fn new(ptr: DevicePtr) -> Self
pub const unsafe fn new(ptr: DevicePtr) -> Self
Creates a mutable device scalar from a raw device pointer.
Safety
ptr must point to a valid writable device allocation containing a T
for every cuSPARSE operation using this scalar.
pub const fn as_device_ptr(&self) -> DevicePtr
pub const fn as_mut_ptr(&self) -> *mut T