Module types

Module types 

Enums

Action
Selects whether an operation processes only indices or both data and indices.
ColorAlgorithm
CsrToCscAlgorithm
DenseToSparseAlgorithm
DiagonalType
Selects whether matrix diagonal entries are treated as unity. The diagonal elements are always assumed to be present, but if DiagonalType::Unit is passed to an operation, the operation assumes that all diagonal entries are one and does not read or modify them. This behavior is independent of the actual values stored in memory.
Direction
Selects whether dense matrix elements are scanned by rows or columns in cusparse[S|D|C|Z]nnz. This also controls block storage format in BSR matrices.
FillMode
Selects whether the lower or upper part of a matrix is stored in sparse storage.
Format
Describes the sparse matrix storage format.
IndexBase
Selects whether matrix indices are zero-based or one-based.
IndexType
Describes the integer type used for sparse matrix indices.
MatrixType
Describes the matrix kind stored in sparse storage. For symmetric, Hermitian, and triangular matrices, only their lower or upper part is assumed to be stored.
Operation
Selects the operation applied to an input, such as a sparse matrix or vector.
Order
Describes the memory layout of a dense matrix.
PointerMode
Indicates whether scalar values are read from host memory or device memory. If an operation uses several scalar values, all of them use the same pointer mode. The pointer mode can be set and retrieved with Context::set_scalar_pointer_mode and Context::scalar_pointer_mode, respectively.
SddmmAlgorithm
SpGemmAlgorithm
SpMmAlgorithm
SpMmOpAlgorithm
SpMvAlgorithm
SpSMAlgorithm
SpSmUpdate
SpSvAlgorithm
SpSvUpdate
SparseMatrixAttribute
SparseToDenseAlgorithm

Traits

IndexTypeLike