#[non_exhaustive]#[repr(u32)]pub enum Format {
Csr = 1,
Csc = 2,
Coo = 3,
BlockedEll = 5,
Bsr = 6,
SlicedEllpack = 7,
}Expand description
Describes the sparse matrix storage format.
Variants (Non-exhaustive)
This enum is marked as non-exhaustive
Non-exhaustive enums could have additional variants added in future. Therefore, when matching against variants of non-exhaustive enums, an extra wildcard arm must be added to account for any future variants.
Csr = 1
The matrix is stored in Compressed Sparse Row (CSR) format.
Csc = 2
The matrix is stored in Compressed Sparse Column (CSC) format.
Coo = 3
The matrix is stored in Coordinate (COO) format organized in Structure of Arrays (SoA) layout.
BlockedEll = 5
The matrix is stored in Blocked-Ellpack (Blocked-ELL) format.
Bsr = 6
The matrix is stored in Block Sparse Row (BSR) format.
SlicedEllpack = 7