#[non_exhaustive]#[repr(u32)]pub enum PointerMode {
Host = 0,
Device = 1,
}Expand description
Indicates whether scalar values are read from host memory or device memory.
If an operation uses several scalar values, all of them use the same pointer mode.
The pointer mode can be set and retrieved with Context::set_scalar_pointer_mode and Context::scalar_pointer_mode, respectively.
Variants (Non-exhaustive)
This enum is marked as non-exhaustive
Non-exhaustive enums could have additional variants added in future. Therefore, when matching against variants of non-exhaustive enums, an extra wildcard arm must be added to account for any future variants.