#[non_exhaustive]#[repr(u32)]pub enum DiagonalType {
NonUnit = 0,
Unit = 1,
}Expand description
Selects whether matrix diagonal entries are treated as unity.
The diagonal elements are always assumed to be present, but if
DiagonalType::Unit is passed to an operation, the operation assumes that
all diagonal entries are one and does not read or modify them.
This behavior is independent of the actual values stored in memory.
Variants (Non-exhaustive)
This enum is marked as non-exhaustive
Non-exhaustive enums could have additional variants added in future. Therefore, when matching against variants of non-exhaustive enums, an extra wildcard arm must be added to account for any future variants.
NonUnit = 0
The matrix diagonal has non-unit elements.
Unit = 1
The matrix diagonal has unit elements.